/*
 * CharacterMapPlugin.java
 * Copyright (C) 2000 Slava Pestov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import javax.swing.border.EmptyBorder;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.Vector;
import org.gjt.sp.jedit.*;


public class CharacterMapPlugin extends EditPlugin
{
	public void start()
	{
		jEdit.addAction(new OpenAction());
	}


	public void createMenuItems(Vector menuItems)
	{
		menuItems.addElement(GUIUtilities.loadMenuItem("character-map"));
	}


	static class OpenAction extends EditAction
	{
		OpenAction()
		{
			super("character-map");
		}

		public void actionPerformed(ActionEvent evt)
		{
			new CharacterMap(getView(evt));
		}
	}


	static class CharacterMap extends JDialog
	{
		CharacterMap(View view)
		{
			super(view,jEdit.getProperty("character-map.title"),false);

			this.view = view;

			JPanel content = new JPanel(new BorderLayout());
			content.setBorder(new EmptyBorder(12,12,12,12));
			setContentPane(content);

			JLabel caption = new JLabel(jEdit.getProperty("character-map.caption"));
			caption.setBorder(new EmptyBorder(0,0,12,0));
			content.add(BorderLayout.NORTH,caption);

			JPanel characters = new JPanel(new GridLayout(16,16,2,2));
			ActionHandler actionHandler = new ActionHandler();

			Insets zero = new Insets(0,0,0,0);

			Font fnt = view.getTextArea().getPainter().getFont();
			FontMetrics fm = view.getTextArea().getPainter().getFontMetrics();
			int max_height = fm.getMaxAscent() + fm.getMaxDescent();
			int max_width = fm.getMaxAdvance();
			Dimension dim = new Dimension(max_width,max_height);

			for(int i = 0; i < 256; i++)
			{
				String tooltip = "Char: " + (char)i + " Int: "
					+ i + " Hex: " + Integer.toString(i,16);
				JButton button = new JButton(String.valueOf((char)i));
				button.setFont(fnt);
				button.setToolTipText(tooltip);
				button.setRequestFocusEnabled(false);
				button.addActionListener(actionHandler);
				button.setMargin(zero);
				button.setPreferredSize(dim);
				characters.add(button);
			}

			content.add(BorderLayout.CENTER,characters);

			pack();
			GUIUtilities.loadGeometry(this, "character-map");
			setDefaultCloseOperation(DISPOSE_ON_CLOSE);
			addKeyListener(new KeyHandler());
			show();
		}


		public void dispose()
		{
			view = null;
			GUIUtilities.saveGeometry(this, "character-map");
			super.dispose();
		}


		// private members
		private View view;


		class ActionHandler implements ActionListener
		{
			public void actionPerformed(ActionEvent evt)
			{
				view.getTextArea().setSelectedText(
					evt.getActionCommand());
			}
		}


		class KeyHandler extends KeyAdapter
		{
			public void keyPressed(KeyEvent evt)
			{
				if(evt.isConsumed())
					return;

				if(evt.getKeyCode() == KeyEvent.VK_ESCAPE)
				{
					dispose();
					evt.consume();
				}
			}
		}
	}
}
